@@ -162,7 +162,6 @@ struct prop_info {
 	STRING_PROP   (E_CONTACT_HOMEPAGE_URL,       "wWWHomePage"),
 	STRING_PROP   (E_CONTACT_FREEBUSY_URL,       "msExchFBURL"),
 	STRING_PROP   (E_CONTACT_NOTE,               "info"), 
-	STRING_PROP   (E_CONTACT_FILE_AS,            "fileAs"),
 
 #undef STRING_PROP
 #undef COMPLEX_PROP
@@ -767,7 +766,6 @@ func_contains(ESExp *f, int argc, ESExpResult **argv, void *data)
 
 	propname = argv[0]->value.string;
 	str = argv[1]->value.string;
-	ldap_attr = query_prop_to_ldap(argv[0]->value.string);
 
 	if (!strcmp(propname, "x-evolution-any-field")) {
 		/* This gui does (contains "x-evolution-any-field" ""),
@@ -778,15 +776,12 @@ func_contains(ESExp *f, int argc, ESExpResult **argv, void *data)
 		if (strlen(str) == 0) {
 			r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 			r->value.bool = FALSE;
-		} else if (!strcmp(propname, "file_as")) {
-			r = e_sexp_result_new(f, ESEXP_RES_STRING);
-			r->value.string = g_strdup_printf ("(|(displayName=%s*)(sn=%s*)(%s=%s*))", 
-							   str, str, ldap_attr, str);
 		} else
 			r = e_sexp_result_new(f, ESEXP_RES_UNDEFINED);
 		return r;
 	}
 
+	ldap_attr = query_prop_to_ldap(argv[0]->value.string);
 	if (!ldap_attr) {
 		/* Attribute doesn't exist, so it can't possibly match */
 		r = e_sexp_result_new(f, ESEXP_RES_BOOL);
@@ -1304,6 +1299,22 @@ get_supported_fields (EBookBackend *backend,
 						  supported_fields);
 }
 
+static void
+get_required_fields (EBookBackend *backend,
+		     EDataBook *book,
+		     guint32 opid)
+{
+	GList *fields = NULL;
+	
+	fields = g_list_append (fields, e_contact_field_name (E_CONTACT_FILE_AS));
+	e_data_book_respond_get_required_fields (book,
+						  opid,
+						  GNOME_Evolution_Addressbook_Success,
+						  fields);
+	g_list_free (fields);
+
+}
+
 static void
 get_supported_auth_methods (EBookBackend *backend,
 			    EDataBook    *book,
@@ -1373,6 +1384,10 @@ get_static_capabilities (EBookBackend *backend)
 
 /**
  * e_book_backend_gal_new:
+ *
+ * Creates a new #EBookBackendGAL.
+ *
+ * Return value: the new #EBookBackendGAL.
  */
 EBookBackend *
 e_book_backend_gal_new (void)
@@ -1453,6 +1468,7 @@ class_init (EBookBackendGALClass *klass)
 	backend_class->get_changes                = get_changes;
 	backend_class->authenticate_user          = authenticate_user;
 	backend_class->get_supported_fields       = get_supported_fields;
+	backend_class->get_required_fields        = get_required_fields;
 	backend_class->get_supported_auth_methods = get_supported_auth_methods;
 	backend_class->cancel_operation           = cancel_operation;
 
