@@ -50,6 +50,12 @@
 #include <stdlib.h>
 #include <string.h>
 
+#define d(x) x
+#define ADS_UF_DONT_EXPIRE_PASSWORD 0x10000
+#define ONE_HUNDRED_NANOSECOND 0.000000100
+#define SECONDS_IN_DAY 86400
+#define PASSWD_EXPIRY_NOTIFICATION_PERIOD 7
+
 struct _ExchangeAccountPrivate {
 	E2kContext *ctx;
 	E2kGlobalCatalog *gc;
@@ -879,6 +885,56 @@ is_password_expired (ExchangeAccount *account, E2kAutoconfig *ac)
 	return (result == E2K_KERBEROS_PASSWORD_EXPIRED);
 }
 
+static void
+find_passwd_exp_period (ExchangeAccount *account, E2kGlobalCatalogEntry *entry)
+{
+	double max_pwd_age = 0;
+	int max_pwd_age_days;
+	E2kOperation gcop;
+	E2kGlobalCatalogStatus gcstatus;
+	const char *passwd_expiry_msg=NULL;
+
+	/* Check for password expiry period */ 
+	/* This needs to be invoked after is_password_expired(), i.e., 
+	   only if password is not expired */
+
+	/* Check for account control value for a user */
+
+	e2k_operation_init (&gcop);
+	gcstatus = e2k_global_catalog_lookup (account->priv->gc, 
+					      &gcop, 
+					      E2K_GLOBAL_CATALOG_LOOKUP_BY_EMAIL, 
+					      account->priv->identity_email, 
+					      E2K_GLOBAL_CATALOG_LOOKUP_ACCOUNT_CONTROL, 
+					      &entry); 
+	e2k_operation_free (&gcop);
+	if (gcstatus != E2K_GLOBAL_CATALOG_OK) 
+		return;
+       
+	if (entry->user_account_control & ADS_UF_DONT_EXPIRE_PASSWORD) 
+		return;         /* Password is not set to expire */
+
+	/* Here we don't check not setting the password and expired password */ 
+	/* Check for the maximum password age set */
+
+	e2k_operation_init (&gcop); 
+	max_pwd_age = lookup_passwd_max_age (account->priv->gc, &gcop); 
+	e2k_operation_free (&gcop);
+
+	if (max_pwd_age > 0) {
+		/* Calculate password expiry period */
+		max_pwd_age_days = 
+			( max_pwd_age * ONE_HUNDRED_NANOSECOND ) / SECONDS_IN_DAY;
+
+		if (max_pwd_age_days <= PASSWD_EXPIRY_NOTIFICATION_PERIOD) {
+			passwd_expiry_msg = 
+			g_strdup_printf ("Your password will expire in next %d days \n", 
+					  max_pwd_age_days);
+			e_notice (NULL, GTK_MESSAGE_WARNING, passwd_expiry_msg); 
+		} 
+	} 
+}
+
 char *
 exchange_account_get_password (ExchangeAccount *account)
 {
@@ -962,6 +1018,10 @@ exchange_account_connect (ExchangeAccount *account)
 	struct dirent *dent;
 	DIR *d;
 	char *old_password, *new_password;
+	E2kGlobalCatalogStatus gcstatus;
+	E2kGlobalCatalogEntry *entry;
+	E2kOperation gcop;
+	const char *quota_msg = NULL;
 	char *user_name = NULL;
 
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
@@ -980,8 +1040,7 @@ exchange_account_connect (ExchangeAccount *account)
 	g_mutex_unlock (account->priv->connect_lock);
 
 	if (account->priv->windows_domain)
-		user_name = g_strdup_printf ("%s\\%s", account->priv->windows_domain, 
-							account->priv->username);
+		user_name = g_strdup_printf ("%s\\%s", account->priv->windows_domain, account->priv->username);
 	else
 		user_name = g_strdup (account->priv->username);
 
@@ -1022,6 +1081,7 @@ exchange_account_connect (ExchangeAccount *account)
 
 		switch (result) {
 		case E2K_AUTOCONFIG_AUTH_ERROR:
+		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_NTLM:
 			errmsg = _("Could not authenticate to server. "
 				   "(Password incorrect?)\n\n");
 			goto try_password_again;
@@ -1037,13 +1097,13 @@ exchange_account_connect (ExchangeAccount *account)
 				   "password wrong.\n\n");
 			goto try_password_again;
 
-		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_NTLM:
-			ac->use_ntlm = 1;
-			goto try_connect_again;
-
 		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_BASIC:
-			ac->use_ntlm = 0;
-			goto try_connect_again;
+			errmsg = _("Could not authenticate to server."
+				   "\n\nYou may need to use Plaintext "
+				   "Password authentication.\n\nOr "
+				   "you might have just typed your "
+				   "password wrong.\n\n");
+			goto try_password_again;
 
 		case E2K_AUTOCONFIG_REDIRECT:
 			if (!redirected && account_moved (account, ac))
@@ -1150,6 +1210,32 @@ exchange_account_connect (ExchangeAccount *account)
 			account->default_timezone = g_strdup (timezone);
 	}
 
+	/* Find the password expiery peripod and display warning */
+	find_passwd_exp_period(account, entry);
+	
+	/* Check for quota warnings */
+	e2k_operation_init (&gcop);
+	gcstatus = e2k_global_catalog_lookup (account->priv->gc, &gcop,
+                                            E2K_GLOBAL_CATALOG_LOOKUP_BY_EMAIL,
+                                            account->priv->identity_email,
+					    E2K_GLOBAL_CATALOG_LOOKUP_QUOTA,
+                                            &entry);	
+	e2k_operation_free (&gcop);
+	
+	if (gcstatus == E2K_GLOBAL_CATALOG_OK) {
+
+		if (entry->quota_norecv) {
+			quota_msg = g_strdup_printf ("You have exceeded your quota for storing mails on this server. Your current usage is : %d . You will not be able to either send or recieve mails now\n", entry->quota_norecv);
+		} else if (entry->quota_nosend) {
+			quota_msg = g_strdup_printf ("You are nearing your quota available for storing mails on this server. Your current usage is : %d . You will not be able to send mails till you clear up some space by deleting some mails.\n", entry->quota_nosend);
+		} else if (entry->quota_warn) {
+			quota_msg = g_strdup_printf ("You are nearing your quota available for storing mails on this server. Your current usage is : %d . Try to clear up some space by deleting some mails.\n", entry->quota_warn);
+		}
+		
+		if (quota_msg)
+			e_notice (NULL, GTK_MESSAGE_INFO, quota_msg);
+	}
+
 	/* Set up Personal Folders hierarchy */
 	phys_uri_prefix = g_strdup_printf ("exchange://%s/personal",
 					   account->priv->uri_authority);
@@ -1519,12 +1605,10 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 	account->priv->password_key = g_strdup_printf ("exchange://%s", account->priv->uri_authority);
 
 	account->priv->username = g_strdup (uri->user);
-
 	if (uri->domain)
 		account->priv->windows_domain = g_strdup (uri->domain);
 	else
 		account->priv->windows_domain = NULL;
-
 	account->exchange_server = g_strdup (uri->host);
 	if (uri->authmech && !strcmp (uri->authmech, "Basic"))
 		account->priv->auth_pref = E2K_AUTOCONFIG_USE_BASIC;
